@@ -23,8 +23,8 @@ from stevedore import driver as stevedore_driver
 from octavia_lib.api.drivers import data_models as driver_dm
 from octavia_lib.api.drivers import exceptions
 from octavia_lib.api.drivers import provider_base as driver_base
-from octavia_lib.common import constants as lib_consts
 
+from octavia.api.drivers.amphora_driver import availability_zone_schema
 from octavia.api.drivers.amphora_driver import flavor_schema
 from octavia.api.drivers import utils as driver_utils
 from octavia.common import constants as consts
@@ -69,13 +69,6 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
                                           project_id=project_id, vip=vip_obj)
 
         network_driver = utils.get_network_driver()
-        vip_network = network_driver.get_network(
-            vip_dictionary[lib_consts.VIP_NETWORK_ID])
-        if not vip_network.port_security_enabled:
-            message = "Port security must be enabled on the VIP network."
-            raise exceptions.DriverError(user_fault_string=message,
-                                         operator_fault_string=message)
-
         try:
             vip = network_driver.allocate_vip(lb_obj)
         except network_base.AllocateVIPException as e:
@@ -94,8 +87,11 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
     def loadbalancer_create(self, loadbalancer):
         if loadbalancer.flavor == driver_dm.Unset:
             loadbalancer.flavor = None
+        if loadbalancer.availability_zone == driver_dm.Unset:
+            loadbalancer.availability_zone = None
         payload = {consts.LOAD_BALANCER_ID: loadbalancer.loadbalancer_id,
-                   consts.FLAVOR: loadbalancer.flavor}
+                   consts.FLAVOR: loadbalancer.flavor,
+                   consts.AVAILABILITY_ZONE: loadbalancer.availability_zone}
         self.client.cast({}, 'create_load_balancer', **payload)
 
     def loadbalancer_delete(self, loadbalancer, cascade=False):
@@ -172,16 +168,16 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
             pool_dict['enabled'] = pool_dict.pop('admin_state_up')
         pool_id = pool_dict.pop('pool_id')
         if 'tls_container_ref' in pool_dict:
-            pool_dict['tls_certificate_id'] = pool_dict.pop(
-                'tls_container_ref')
+            pool_dict['tls_container_id'] = pool_dict.pop('tls_container_ref')
         pool_dict.pop('tls_container_data', None)
         if 'ca_tls_container_ref' in pool_dict:
             pool_dict['ca_tls_certificate_id'] = pool_dict.pop(
                 'ca_tls_container_ref')
         pool_dict.pop('ca_tls_container_data', None)
-        if 'crl_container_ref' in pool_dict:
-            pool_dict['crl_container_id'] = pool_dict.pop('crl_container_ref')
-        pool_dict.pop('crl_container_data', None)
+        if 'client_crl_container_ref' in pool_dict:
+            pool_dict['client_crl_container_id'] = pool_dict.pop(
+                'client_crl_container_ref')
+        pool_dict.pop('client_crl_container_data', None)
 
         payload = {consts.POOL_ID: pool_id,
                    consts.POOL_UPDATES: pool_dict}
@@ -212,9 +208,7 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
                    consts.MEMBER_UPDATES: member_dict}
         self.client.cast({}, 'update_member', **payload)
 
-    def member_batch_update(self, members):
-        # Get a list of existing members
-        pool_id = members[0].pool_id
+    def member_batch_update(self, pool_id, members):
         # The DB should not have updated yet, so we can still use the pool
         db_pool = self.repositories.pool.get(db_apis.get_session(), id=pool_id)
 
@@ -247,10 +241,13 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
             if m.id not in new_member_ids:
                 deleted_members.append(m)
 
-        payload = {'old_member_ids': [m.id for m in deleted_members],
-                   'new_member_ids': [m.member_id for m in new_members],
-                   'updated_members': updated_members}
-        self.client.cast({}, 'batch_update_members', **payload)
+        if deleted_members or new_members or updated_members:
+            payload = {'old_member_ids': [m.id for m in deleted_members],
+                       'new_member_ids': [m.member_id for m in new_members],
+                       'updated_members': updated_members}
+            self.client.cast({}, 'batch_update_members', **payload)
+        else:
+            LOG.info("Member batch update is a noop, returning early.")
 
     def _validate_members(self, db_pool, members):
         if db_pool.protocol == consts.PROTOCOL_UDP:
@@ -395,3 +392,70 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
             # TODO(johnsom) Fix this to raise a NotFound error
             # when the octavia-lib supports it.
             compute_driver.validate_flavor(compute_flavor)
+
+    # Availability Zone
+    def get_supported_availability_zone_metadata(self):
+        """Returns the valid availability zone metadata keys and descriptions.
+
+        This extracts the valid availability zone metadata keys and
+        descriptions from the JSON validation schema and returns it as a
+        dictionary.
+
+        :return: Dictionary of availability zone metadata keys and descriptions
+        :raises DriverError: An unexpected error occurred.
+        """
+        try:
+            props = (
+                availability_zone_schema.SUPPORTED_AVAILABILITY_ZONE_SCHEMA[
+                    'properties'])
+            return {k: v.get('description', '') for k, v in props.items()}
+        except Exception as e:
+            raise exceptions.DriverError(
+                user_fault_string='Failed to get the supported availability '
+                                  'zone metadata due to: {}'.format(str(e)),
+                operator_fault_string='Failed to get the supported '
+                                      'availability zone metadata due to: '
+                                      '{}'.format(str(e)))
+
+    def validate_availability_zone(self, availability_zone_dict):
+        """Validates availability zone profile data.
+
+        This will validate an availability zone profile dataset against the
+        availability zone settings the amphora driver supports.
+
+        :param availability_zone_dict: The availability zone dict to validate.
+        :type availability_zone_dict: dict
+        :return: None
+        :raises DriverError: An unexpected error occurred.
+        :raises UnsupportedOptionError: If the driver does not support
+          one of the availability zone settings.
+        """
+        try:
+            validate(
+                availability_zone_dict,
+                availability_zone_schema.SUPPORTED_AVAILABILITY_ZONE_SCHEMA)
+        except js_exceptions.ValidationError as e:
+            error_object = ''
+            if e.relative_path:
+                error_object = '{} '.format(e.relative_path[0])
+            raise exceptions.UnsupportedOptionError(
+                user_fault_string='{0}{1}'.format(error_object, e.message),
+                operator_fault_string=str(e))
+        except Exception as e:
+            raise exceptions.DriverError(
+                user_fault_string='Failed to validate the availability zone '
+                                  'metadata due to: {}'.format(str(e)),
+                operator_fault_string='Failed to validate the availability '
+                                      'zone metadata due to: {}'.format(str(e))
+            )
+        compute_zone = availability_zone_dict.get(consts.COMPUTE_ZONE, None)
+        if compute_zone:
+            compute_driver = stevedore_driver.DriverManager(
+                namespace='octavia.compute.drivers',
+                name=CONF.controller_worker.compute_driver,
+                invoke_on_load=True
+            ).driver
+
+            # TODO(johnsom) Fix this to raise a NotFound error
+            # when the octavia-lib supports it.
+            compute_driver.validate_availability_zone(compute_zone)
