@@ -22,6 +22,7 @@ from oslo_log import log as logging
 from oslo_utils import excutils
 from stevedore import driver as stevedore_driver
 
+from octavia.common import constants
 from octavia.common import data_models
 from octavia.common import exceptions
 from octavia.common.tls_utils import cert_parser
@@ -220,19 +221,6 @@ def listener_dict_to_provider_dict(listener_dict, for_delete=False):
     if 'tls_certificate_id' in new_listener_dict:
         new_listener_dict['default_tls_container_ref'] = new_listener_dict.pop(
             'tls_certificate_id')
-    if 'sni_containers' in new_listener_dict:
-        sni_refs = []
-        sni_containers = new_listener_dict.pop('sni_containers')
-        for sni in sni_containers:
-            if 'tls_container_id' in sni:
-                sni_refs.append(sni['tls_container_id'])
-            else:
-                raise exceptions.ValidationException(
-                    detail=_('Invalid SNI container on listener'))
-        new_listener_dict['sni_container_refs'] = sni_refs
-    if 'sni_container_refs' in listener_dict:
-        listener_dict['sni_containers'] = listener_dict.pop(
-            'sni_container_refs')
     if 'client_ca_tls_certificate_id' in new_listener_dict:
         new_listener_dict['client_ca_tls_container_ref'] = (
             new_listener_dict.pop('client_ca_tls_certificate_id'))
@@ -245,8 +233,6 @@ def listener_dict_to_provider_dict(listener_dict, for_delete=False):
         SNI_objs = []
         for sni in listener_obj.sni_containers:
             if isinstance(sni, dict):
-                if 'listener' in sni:
-                    del sni['listener']
                 sni_obj = data_models.SNI(**sni)
                 SNI_objs.append(sni_obj)
             elif isinstance(sni, str):
@@ -297,6 +283,20 @@ def listener_dict_to_provider_dict(listener_dict, for_delete=False):
         new_listener_dict['allowed_cidrs'] = [cidr_dict['cidr'] for
                                               cidr_dict in cidrs_dict_list]
 
+    # Format the sni_containers -> sni_container_refs
+    sni_containers = new_listener_dict.pop('sni_containers', None)
+    if sni_containers:
+        new_listener_dict['sni_container_refs'] = []
+        for sni in sni_containers:
+            if isinstance(sni, dict):
+                new_listener_dict['sni_container_refs'].append(
+                    sni['tls_container_id'])
+            elif isinstance(sni, str):
+                new_listener_dict['sni_container_refs'].append(sni)
+            else:
+                raise exceptions.ValidationException(
+                    detail=_('Invalid SNI container on listener'))
+
     # Remove the DB back references
     if 'load_balancer' in new_listener_dict:
         del new_listener_dict['load_balancer']
@@ -544,6 +544,9 @@ def vip_dict_to_provider_dict(vip_dict):
         new_vip_dict['vip_subnet_id'] = vip_dict['subnet_id']
     if 'qos_policy_id' in vip_dict:
         new_vip_dict['vip_qos_policy_id'] = vip_dict['qos_policy_id']
+    if constants.OCTAVIA_OWNED in vip_dict:
+        new_vip_dict[constants.OCTAVIA_OWNED] = vip_dict[
+            constants.OCTAVIA_OWNED]
     return new_vip_dict
 
 
@@ -559,4 +562,6 @@ def provider_vip_dict_to_vip_obj(vip_dictionary):
         vip_obj.subnet_id = vip_dictionary['vip_subnet_id']
     if 'vip_qos_policy_id' in vip_dictionary:
         vip_obj.qos_policy_id = vip_dictionary['vip_qos_policy_id']
+    if constants.OCTAVIA_OWNED in vip_dictionary:
+        vip_obj.octavia_owned = vip_dictionary[constants.OCTAVIA_OWNED]
     return vip_obj
