@@ -63,6 +63,7 @@ class ListenerResponse(BaseListenerType):
     client_authentication = wtypes.wsattr(wtypes.StringType())
     client_crl_container_ref = wtypes.wsattr(wtypes.StringType())
     allowed_cidrs = wtypes.wsattr([types.CidrType()])
+    tls_ciphers = wtypes.StringType()
 
     @classmethod
     def from_data_model(cls, data_model, children=False):
@@ -151,6 +152,7 @@ class ListenerPOST(BaseListenerType):
         default=constants.CLIENT_AUTH_NONE)
     client_crl_container_ref = wtypes.StringType(max_length=255)
     allowed_cidrs = wtypes.wsattr([types.CidrType()])
+    tls_ciphers = wtypes.StringType(max_length=2048)
 
 
 class ListenerRootPOST(types.BaseType):
@@ -188,6 +190,7 @@ class ListenerPUT(BaseListenerType):
         wtypes.Enum(str, *constants.SUPPORTED_CLIENT_AUTH_MODES))
     client_crl_container_ref = wtypes.StringType(max_length=255)
     allowed_cidrs = wtypes.wsattr([types.CidrType()])
+    tls_ciphers = wtypes.StringType(max_length=2048)
 
 
 class ListenerRootPUT(types.BaseType):
@@ -238,6 +241,7 @@ class ListenerSingleCreate(BaseListenerType):
         default=constants.CLIENT_AUTH_NONE)
     client_crl_container_ref = wtypes.StringType(max_length=255)
     allowed_cidrs = wtypes.wsattr([types.CidrType()])
+    tls_ciphers = wtypes.StringType(max_length=2048)
 
 
 class ListenerStatusResponse(BaseListenerType):
