@@ -293,12 +293,14 @@ SUPPORTED_TASKFLOW_ENGINE_TYPES = ['serial', 'parallel']
 
 # Task/Flow constants
 ACTIVE_CONNECTIONS = 'active_connections'
+ADD_NICS = 'add_nics'
 ADDED_PORTS = 'added_ports'
 ADMIN_STATE_UP = 'admin_state_up'
 ALLOWED_ADDRESS_PAIRS = 'allowed_address_pairs'
 AMP_DATA = 'amp_data'
 AMP_VRRP_INT = 'amp_vrrp_int'
 AMPHORA = 'amphora'
+AMPHORA_DICT = 'amphora_dict'
 AMPHORA_ID = 'amphora_id'
 AMPHORA_INDEX = 'amphora_index'
 AMPHORA_NETWORK_CONFIG = 'amphora_network_config'
@@ -310,16 +312,20 @@ ATTEMPT_NUMBER = 'attempt_number'
 BASE_PORT = 'base_port'
 BYTES_IN = 'bytes_in'
 BYTES_OUT = 'bytes_out'
+CACHED_ZONE = 'cached_zone'
 CA_TLS_CERTIFICATE_ID = 'ca_tls_certificate_id'
 CIDR = 'cidr'
 CLIENT_CA_TLS_CERTIFICATE_ID = 'client_ca_tls_certificate_id'
 CLIENT_CRL_CONTAINER_ID = 'client_crl_container_id'
 COMPUTE_ID = 'compute_id'
 COMPUTE_OBJ = 'compute_obj'
+COMPUTE_ZONE = 'compute_zone'
 CONN_MAX_RETRIES = 'conn_max_retries'
 CONN_RETRY_INTERVAL = 'conn_retry_interval'
 CREATED_AT = 'created_at'
 CRL_CONTAINER_ID = 'crl_container_id'
+DEFAULT_TLS_CONTAINER_DATA = 'default_tls_container_data'
+DELETE_NICS = 'delete_nics'
 DELTA = 'delta'
 DELTAS = 'deltas'
 DESCRIPTION = 'description'
@@ -332,13 +338,18 @@ FAILOVER_AMPHORA_ID = 'failover_amphora_id'
 FIELDS = 'fields'
 FIXED_IPS = 'fixed_ips'
 FLAVOR_ID = 'flavor_id'
+HA_IP = 'ha_ip'
+HA_PORT_ID = 'ha_port_id'
 HEALTH_MON = 'health_mon'
 HEALTH_MONITOR = 'health_monitor'
 HEALTH_MONITOR_ID = 'health_monitor_id'
+HEALTHMONITOR_ID = 'healthmonitor_id'
 HEALTH_MONITOR_UPDATES = 'health_monitor_updates'
 ID = 'id'
+IMAGE_ID = 'image_id'
 IP_ADDRESS = 'ip_address'
 IPV6_ICMP = 'ipv6-icmp'
+LB_NETWORK_IP = 'lb_network_ip'
 L7POLICY = 'l7policy'
 L7POLICY_ID = 'l7policy_id'
 L7POLICY_UPDATES = 'l7policy_updates'
@@ -352,15 +363,24 @@ LOADBALANCER = 'loadbalancer'
 LOADBALANCER_ID = 'loadbalancer_id'
 LOAD_BALANCER_ID = 'load_balancer_id'
 LOAD_BALANCER_UPDATES = 'load_balancer_updates'
+MANAGEMENT_NETWORK = 'management_network'
 MEMBER = 'member'
 MEMBER_ID = 'member_id'
 MEMBER_PORTS = 'member_ports'
 MEMBER_UPDATES = 'member_updates'
 MESSAGE = 'message'
 NAME = 'name'
+NETWORK = 'network'
 NETWORK_ID = 'network_id'
 NICS = 'nics'
 OBJECT = 'object'
+ORIGINAL_HEALTH_MONITOR = 'original_health_monitor'
+ORIGINAL_L7POLICY = 'original_l7policy'
+ORIGINAL_L7RULE = 'original_l7rule'
+ORIGINAL_LISTENER = 'original_listener'
+ORIGINAL_LOADBALANCER = 'original_load_balancer'
+ORIGINAL_MEMBER = 'original_member'
+ORIGINAL_POOL = 'original_pool'
 PASSIVE_FAILURE = 'passive_failure'
 PEER_PORT = 'peer_port'
 POOL = 'pool'
@@ -378,16 +398,18 @@ REDIRECT_POOL = 'redirect_pool'
 REQ_CONN_TIMEOUT = 'req_conn_timeout'
 REQ_READ_TIMEOUT = 'req_read_timeout'
 REQUEST_ERRORS = 'request_errors'
+ROLE = 'role'
 SECURITY_GROUPS = 'security_groups'
 SECURITY_GROUP_RULES = 'security_group_rules'
 SERVER_GROUP_ID = 'server_group_id'
 SERVER_PEM = 'server_pem'
+SNI_CONTAINER_DATA = 'sni_container_data'
 SNI_CONTAINERS = 'sni_containers'
 SOFT_ANTI_AFFINITY = 'soft-anti-affinity'
+STATUS = 'status'
 STATUS_CODE = 'status_code'
 SUBNET = 'subnet'
 SUBNET_ID = 'subnet_id'
-SUBNET = 'subnet'
 TAGS = 'tags'
 TENANT_ID = 'tenant_id'
 TIMEOUT_DICT = 'timeout_dict'
@@ -397,13 +419,21 @@ TOPOLOGY = 'topology'
 TOTAL_CONNECTIONS = 'total_connections'
 UPDATED_AT = 'updated_at'
 UPDATE_DICT = 'update_dict'
+VALID_VIP_NETWORKS = 'valid_vip_networks'
 VIP = 'vip'
 VIP_ADDRESS = 'vip_address'
 VIP_NETWORK = 'vip_network'
+VIP_PORT_ID = 'vip_port_id'
+VIP_QOS_POLICY_ID = 'vip_qos_policy_id'
 VIP_SG_ID = 'vip_sg_id'
 VIP_SUBNET = 'vip_subnet'
+VIP_SUBNET_ID = 'vip_subnet_id'
+VRRP_ID = 'vrrp_id'
+VRRP_IP = 'vrrp_ip'
 VRRP_GROUP = 'vrrp_group'
 VRRP_PORT = 'vrrp_port'
+VRRP_PORT_ID = 'vrrp_port_id'
+VRRP_PRIORITY = 'vrrp_priority'
 
 # Taskflow flow and task names
 CERT_ROTATE_AMPHORA_FLOW = 'octavia-cert-rotate-amphora-flow'
@@ -433,6 +463,7 @@ DELETE_L7POLICY_FLOW = 'octavia-delete-l7policy-flow'
 DELETE_L7RULE_FLOW = 'octavia-delete-l7policy-flow'
 FAILOVER_AMPHORA_FLOW = 'octavia-failover-amphora-flow'
 FAILOVER_LOADBALANCER_FLOW = 'octavia-failover-loadbalancer-flow'
+FINALIZE_AMPHORA_FLOW = 'octavia-finalize-amphora-flow'
 LOADBALANCER_NETWORKING_SUBFLOW = 'octavia-new-loadbalancer-net-subflow'
 UPDATE_HEALTH_MONITOR_FLOW = 'octavia-update-health-monitor-flow'
 UPDATE_LISTENER_FLOW = 'octavia-update-listener-flow'
@@ -614,8 +645,6 @@ HAPROXY_DEFAULT_MAXCONN = 50000
 # the sysctl fs.file-max fs.nr_open settings in the image
 HAPROXY_MAX_MAXCONN = 1000000
 
-RESTARTING = 'RESTARTING'
-
 # Quota Constants
 QUOTA_UNLIMITED = -1
 MIN_QUOTA = QUOTA_UNLIMITED
@@ -714,8 +743,13 @@ RBAC_QUOTA = '{}:quota:'.format(LOADBALANCER_API)
 RBAC_AMPHORA = '{}:amphora:'.format(LOADBALANCER_API)
 RBAC_PROVIDER = '{}:provider:'.format(LOADBALANCER_API)
 RBAC_PROVIDER_FLAVOR = '{}:provider-flavor:'.format(LOADBALANCER_API)
+RBAC_PROVIDER_AVAILABILITY_ZONE = '{}:provider-availability-zone:'.format(
+    LOADBALANCER_API)
 RBAC_FLAVOR = '{}:flavor:'.format(LOADBALANCER_API)
 RBAC_FLAVOR_PROFILE = '{}:flavor-profile:'.format(LOADBALANCER_API)
+RBAC_AVAILABILITY_ZONE = '{}:availability-zone:'.format(LOADBALANCER_API)
+RBAC_AVAILABILITY_ZONE_PROFILE = '{}:availability-zone-profile:'.format(
+    LOADBALANCER_API)
 RBAC_POST = 'post'
 RBAC_PUT = 'put'
 RBAC_PUT_CONFIG = 'put_config'
@@ -730,6 +764,7 @@ RBAC_GET_STATUS = 'get_status'
 
 # PROVIDERS
 OCTAVIA = 'octavia'
+AMPHORAV2 = 'amphorav2'
 
 # systemctl commands
 DISABLE = 'disable'
@@ -741,10 +776,12 @@ AMP_NETNS_SVC_PREFIX = 'amphora-netns'
 # Amphora Feature Compatibility
 HTTP_REUSE = 'has_http_reuse'
 
-# TODO(johnsom) convert this to octavia_lib constant flavor
+# TODO(johnsom) convert these to octavia_lib constants
 # once octavia is transitioned to use octavia_lib
 FLAVOR = 'flavor'
 FLAVOR_DATA = 'flavor_data'
+AVAILABILITY_ZONE = 'availability_zone'
+AVAILABILITY_ZONE_DATA = 'availability_zone_data'
 
 # Flavor metadata
 LOADBALANCER_TOPOLOGY = 'loadbalancer_topology'
@@ -783,6 +820,18 @@ CINDER_STATUS_AVAILABLE = 'available'
 CINDER_STATUS_ERROR = 'error'
 CINDER_ACTION_CREATE_VOLUME = 'create volume'
 
+# The nil UUID (used in octavia for deleted references) - RFC 4122
+NIL_UUID = '00000000-0000-0000-0000-000000000000'
+
+# OpenSSL cipher strings
+CIPHERS_OWASP_SUITE_B = ('TLS_AES_256_GCM_SHA384:TLS_CHACHA20_POLY1305_SHA256:'
+                         'TLS_AES_128_GCM_SHA256:DHE-RSA-AES256-GCM-SHA384:'
+                         'DHE-RSA-AES128-GCM-SHA256:'
+                         'ECDHE-RSA-AES256-GCM-SHA384:'
+                         'ECDHE-RSA-AES128-GCM-SHA256:DHE-RSA-AES256-SHA256:'
+                         'DHE-RSA-AES128-SHA256:ECDHE-RSA-AES256-SHA384:'
+                         'ECDHE-RSA-AES128-SHA256')
+
 VIP_SECURITY_GROUP_PREFIX = 'lb-'
 
 AMP_BASE_PORT_PREFIX = 'octavia-lb-vrrp-'
