@@ -14,16 +14,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import datetime
 import re
 
-from oslo_log import log as logging
-import six
 from sqlalchemy.orm import collections
 
 from octavia.common import constants
 
-LOG = logging.getLogger(__name__)
-
 
 class BaseDataModel(object):
     def to_dict(self, calling_classes=None, recurse=False, **kwargs):
@@ -38,7 +35,11 @@ class BaseDataModel(object):
                 # tags is a list, it doesn't need recurse
                 ret[attr] = value
                 continue
-
+            # We need to have json convertible data for storing it in
+            # persistence jobboard backend.
+            if isinstance(value, datetime.datetime):
+                ret[attr] = value.isoformat()
+                continue
             if recurse:
                 if isinstance(getattr(self, attr), list):
                     ret[attr] = []
@@ -66,8 +67,6 @@ class BaseDataModel(object):
                             recurse=recurse)
                     else:
                         ret[attr] = None
-                elif six.PY2 and isinstance(value, six.text_type):
-                    ret[attr.encode('utf8')] = value.encode('utf8')
                 else:
                     ret[attr] = value
             else:
@@ -279,7 +278,7 @@ class Pool(BaseDataModel):
                  created_at=None, updated_at=None, provisioning_status=None,
                  tags=None, tls_certificate_id=None,
                  ca_tls_certificate_id=None, crl_container_id=None,
-                 tls_enabled=None):
+                 tls_enabled=None, tls_ciphers=None):
         self.id = id
         self.project_id = project_id
         self.name = name
@@ -303,6 +302,7 @@ class Pool(BaseDataModel):
         self.ca_tls_certificate_id = ca_tls_certificate_id
         self.crl_container_id = crl_container_id
         self.tls_enabled = tls_enabled
+        self.tls_ciphers = tls_ciphers
 
     def update(self, update_dict):
         for key, value in update_dict.items():
@@ -329,11 +329,7 @@ class Pool(BaseDataModel):
                     break
         for pool in self.load_balancer.pools:
             if pool.id == self.id:
-                try:
-                    self.load_balancer.pools.remove(pool)
-                except ValueError:
-                    LOG.debug("Pool %s has already been removed from load "
-                              "balancer pools list.", pool.id)
+                self.load_balancer.pools.remove(pool)
                 break
         for l7policy in self.l7policies:
             if l7policy.redirect_pool_id == self.id:
@@ -394,7 +390,7 @@ class Listener(BaseDataModel):
                  timeout_member_data=None, timeout_tcp_inspect=None,
                  tags=None, client_ca_tls_certificate_id=None,
                  client_authentication=None, client_crl_container_id=None,
-                 allowed_cidrs=None):
+                 allowed_cidrs=None, tls_ciphers=None):
         self.id = id
         self.project_id = project_id
         self.name = name
@@ -427,6 +423,7 @@ class Listener(BaseDataModel):
         self.client_authentication = client_authentication
         self.client_crl_container_id = client_crl_container_id
         self.allowed_cidrs = allowed_cidrs or []
+        self.tls_ciphers = tls_ciphers
 
     def update(self, update_dict):
         for key, value in update_dict.items():
@@ -468,7 +465,7 @@ class LoadBalancer(BaseDataModel):
                  topology=None, vip=None, listeners=None, amphorae=None,
                  pools=None, vrrp_group=None, server_group_id=None,
                  created_at=None, updated_at=None, provider=None, tags=None,
-                 flavor_id=None):
+                 flavor_id=None, availability_zone=None):
 
         self.id = id
         self.project_id = project_id
@@ -489,6 +486,7 @@ class LoadBalancer(BaseDataModel):
         self.provider = provider
         self.tags = tags or []
         self.flavor_id = flavor_id
+        self.availability_zone = availability_zone
 
     def update(self, update_dict):
         for key, value in update_dict.items():
@@ -796,6 +794,26 @@ class FlavorProfile(BaseDataModel):
         self.flavor_data = flavor_data
 
 
+class AvailabilityZone(BaseDataModel):
+
+    def __init__(self, name=None, description=None, enabled=None,
+                 availability_zone_profile_id=None):
+        self.name = name
+        self.description = description
+        self.enabled = enabled
+        self.availability_zone_profile_id = availability_zone_profile_id
+
+
+class AvailabilityZoneProfile(BaseDataModel):
+
+    def __init__(self, id=None, name=None, provider_name=None,
+                 availability_zone_data=None):
+        self.id = id
+        self.name = name
+        self.provider_name = provider_name
+        self.availability_zone_data = availability_zone_data
+
+
 class ListenerCidr(BaseDataModel):
 
     def __init__(self, listener_id=None, cidr=None):
