@@ -89,6 +89,7 @@ class VirtualMachineManager(compute_base.ComputeBase):
         self.manager = self._nova_client.servers
         self.server_groups = self._nova_client.server_groups
         self.flavor_manager = self._nova_client.flavors
+        self.availability_zone_manager = self._nova_client.availability_zones
         self.volume_driver = stevedore_driver.DriverManager(
             namespace='octavia.volume.drivers',
             name=CONF.controller_worker.volume_driver,
@@ -99,7 +100,7 @@ class VirtualMachineManager(compute_base.ComputeBase):
               image_id=None, image_tag=None, image_owner=None,
               key_name=None, sec_groups=None, network_ids=None,
               port_ids=None, config_drive_files=None, user_data=None,
-              server_group_id=None):
+              server_group_id=None, availability_zone=None):
         '''Create a new virtual machine.
 
         :param name: optional name for amphora
@@ -122,6 +123,7 @@ class VirtualMachineManager(compute_base.ComputeBase):
                           well or a string
         :param server_group_id: Optional server group id(uuid) which is used
                                 for anti_affinity feature
+        :param availability_zone: Name of the compute availability zone.
 
         :raises ComputeBuildException: if nova failed to build virtual machine
         :returns: UUID of amphora
@@ -140,6 +142,7 @@ class VirtualMachineManager(compute_base.ComputeBase):
 
             server_group = None if server_group_id is None else {
                 "group": server_group_id}
+            az_name = availability_zone or CONF.nova.availability_zone
 
             image_id = _get_image_uuid(
                 self._glance_client, image_id, image_tag, image_owner)
@@ -175,7 +178,7 @@ class VirtualMachineManager(compute_base.ComputeBase):
                 userdata=user_data,
                 config_drive=True,
                 scheduler_hints=server_group,
-                availability_zone=CONF.nova.availability_zone
+                availability_zone=az_name
             )
 
             return amphora.id
@@ -215,10 +218,11 @@ class VirtualMachineManager(compute_base.ComputeBase):
             raise exceptions.ComputeStatusException()
         return constants.DOWN
 
-    def get_amphora(self, compute_id):
+    def get_amphora(self, compute_id, management_network_id=None):
         '''Retrieve the information in nova of a virtual machine.
 
-        :param amphora_id: virtual machine UUID
+        :param compute_id: virtual machine UUID
+        :param management_network_id: ID of the management network
         :returns: an amphora object
         :returns: fault message or None
         '''
@@ -228,12 +232,13 @@ class VirtualMachineManager(compute_base.ComputeBase):
         except Exception:
             LOG.exception("Error retrieving nova virtual machine.")
             raise exceptions.ComputeGetException()
-        return self._translate_amphora(amphora)
+        return self._translate_amphora(amphora, management_network_id)
 
-    def _translate_amphora(self, nova_response):
+    def _translate_amphora(self, nova_response, management_network_id=None):
         '''Convert a nova virtual machine into an amphora object.
 
         :param nova_response: JSON response from nova
+        :param management_network_id: ID of the management network
         :returns: an amphora object
         :returns: fault message or None
         '''
@@ -243,19 +248,19 @@ class VirtualMachineManager(compute_base.ComputeBase):
         lb_network_ip = None
         availability_zone = None
         image_id = None
-        fault = None
+
+        if management_network_id:
+            boot_networks = [management_network_id]
+        else:
+            boot_networks = CONF.controller_worker.amp_boot_network_list
 
         try:
             inf_list = nova_response.interface_list()
-            no_boot_networks = (
-                not CONF.controller_worker.amp_boot_network_list)
             for interface in inf_list:
                 net_id = interface.net_id
-                is_boot_network = (
-                    net_id in CONF.controller_worker.amp_boot_network_list)
                 # Pick the first fixed_ip if this is a boot network or if
                 # there are no boot networks configured (use default network)
-                if is_boot_network or no_boot_networks:
+                if net_id in boot_networks or not boot_networks:
                     lb_network_ip = interface.fixed_ips[0]['ip_address']
                     break
             try:
@@ -424,3 +429,24 @@ class VirtualMachineManager(compute_base.ComputeBase):
             LOG.exception('Nova reports a failure getting flavor details for '
                           'flavor ID %s: %s', flavor_id, e)
             raise
+
+    def validate_availability_zone(self, availability_zone):
+        """Validates that an availability zone exists in nova.
+
+        :param availability_zone: Name of the availability zone to lookup.
+        :raises: NotFound
+        :returns: None
+        """
+        try:
+            compute_zones = [
+                a.zoneName for a in self.availability_zone_manager.list(
+                    detailed=False)]
+            if availability_zone not in compute_zones:
+                LOG.info('Availability zone %s was not found in nova. %s',
+                         availability_zone, compute_zones)
+                raise exceptions.InvalidSubresource(
+                    resource='Nova availability zone', id=availability_zone)
+        except Exception as e:
+            LOG.exception('Nova reports a failure getting listing '
+                          'availability zones: %s', e)
+            raise
