@@ -67,6 +67,7 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         self._l7policy_repo = repo.L7PolicyRepository()
         self._l7rule_repo = repo.L7RuleRepository()
         self._flavor_repo = repo.FlavorRepository()
+        self._az_repo = repo.AvailabilityZoneRepository()
 
         super(ControllerWorker, self).__init__()
 
@@ -84,7 +85,7 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
 
         return repo.get(db_apis.get_session(), id=id)
 
-    def create_amphora(self):
+    def create_amphora(self, availability_zone=None):
         """Creates an Amphora.
 
         This is used to create spare amphora.
@@ -95,7 +96,12 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
             store = {constants.BUILD_TYPE_PRIORITY:
                      constants.LB_CREATE_SPARES_POOL_PRIORITY,
                      constants.FLAVOR: None,
-                     constants.SERVER_GROUP_ID: None}
+                     constants.SERVER_GROUP_ID: None,
+                     constants.AVAILABILITY_ZONE: None}
+            if availability_zone:
+                store[constants.AVAILABILITY_ZONE] = (
+                    self._az_repo.get_availability_zone_metadata_dict(
+                        db_apis.get_session(), availability_zone))
             create_amp_tf = self._taskflow_load(
                 self._amphora_flows.get_create_amphora_flow(),
                 store=store)
@@ -300,7 +306,8 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
             CONF.haproxy_amphora.api_db_commit_retry_max),
         stop=tenacity.stop_after_attempt(
             CONF.haproxy_amphora.api_db_commit_retry_attempts))
-    def create_load_balancer(self, load_balancer_id, flavor=None):
+    def create_load_balancer(self, load_balancer_id, flavor=None,
+                             availability_zone=None):
         """Creates a load balancer by allocating Amphorae.
 
         First tries to allocate an existing Amphora in READY state.
@@ -322,7 +329,8 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         store = {constants.LOADBALANCER_ID: load_balancer_id,
                  constants.BUILD_TYPE_PRIORITY:
                  constants.LB_CREATE_NORMAL_PRIORITY,
-                 constants.FLAVOR: flavor}
+                 constants.FLAVOR: flavor,
+                 constants.AVAILABILITY_ZONE: availability_zone}
 
         topology = lb.topology
 
@@ -424,14 +432,21 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         listeners = pool.listeners
         load_balancer = pool.load_balancer
 
-        create_member_tf = self._taskflow_load(self._member_flows.
-                                               get_create_member_flow(),
-                                               store={constants.MEMBER: member,
-                                                      constants.LISTENERS:
-                                                          listeners,
-                                                      constants.LOADBALANCER:
-                                                          load_balancer,
-                                                      constants.POOL: pool})
+        store = {
+            constants.MEMBER: member,
+            constants.LISTENERS: listeners,
+            constants.LOADBALANCER: load_balancer,
+            constants.POOL: pool}
+        if load_balancer.availability_zone:
+            store[constants.AVAILABILITY_ZONE] = (
+                self._az_repo.get_availability_zone_metadata_dict(
+                    db_apis.get_session(), load_balancer.availability_zone))
+        else:
+            store[constants.AVAILABILITY_ZONE] = {}
+
+        create_member_tf = self._taskflow_load(
+            self._member_flows.get_create_member_flow(),
+            store=store)
         with tf_logging.DynamicLoggingListener(create_member_tf,
                                                log=LOG):
             create_member_tf.run()
@@ -449,10 +464,21 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         listeners = pool.listeners
         load_balancer = pool.load_balancer
 
+        store = {
+            constants.MEMBER: member,
+            constants.LISTENERS: listeners,
+            constants.LOADBALANCER: load_balancer,
+            constants.POOL: pool}
+        if load_balancer.availability_zone:
+            store[constants.AVAILABILITY_ZONE] = (
+                self._az_repo.get_availability_zone_metadata_dict(
+                    db_apis.get_session(), load_balancer.availability_zone))
+        else:
+            store[constants.AVAILABILITY_ZONE] = {}
+
         delete_member_tf = self._taskflow_load(
             self._member_flows.get_delete_member_flow(),
-            store={constants.MEMBER: member, constants.LISTENERS: listeners,
-                   constants.LOADBALANCER: load_balancer, constants.POOL: pool}
+            store=store
         )
         with tf_logging.DynamicLoggingListener(delete_member_tf,
                                                log=LOG):
@@ -490,12 +516,21 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         listeners = pool.listeners
         load_balancer = pool.load_balancer
 
+        store = {
+            constants.LISTENERS: listeners,
+            constants.LOADBALANCER: load_balancer,
+            constants.POOL: pool}
+        if load_balancer.availability_zone:
+            store[constants.AVAILABILITY_ZONE] = (
+                self._az_repo.get_availability_zone_metadata_dict(
+                    db_apis.get_session(), load_balancer.availability_zone))
+        else:
+            store[constants.AVAILABILITY_ZONE] = {}
+
         batch_update_members_tf = self._taskflow_load(
             self._member_flows.get_batch_update_members_flow(
                 old_members, new_members, updated_members),
-            store={constants.LISTENERS: listeners,
-                   constants.LOADBALANCER: load_balancer,
-                   constants.POOL: pool})
+            store=store)
         with tf_logging.DynamicLoggingListener(batch_update_members_tf,
                                                log=LOG):
             batch_update_members_tf.run()
@@ -508,7 +543,6 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         :returns: None
         :raises MemberNotFound: The referenced member was not found
         """
-        member = None
         try:
             member = self._get_db_obj_until_pending_update(
                 self._member_repo, member_id)
@@ -524,17 +558,22 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         listeners = pool.listeners
         load_balancer = pool.load_balancer
 
-        update_member_tf = self._taskflow_load(self._member_flows.
-                                               get_update_member_flow(),
-                                               store={constants.MEMBER: member,
-                                                      constants.LISTENERS:
-                                                          listeners,
-                                                      constants.LOADBALANCER:
-                                                          load_balancer,
-                                                      constants.POOL:
-                                                          pool,
-                                                      constants.UPDATE_DICT:
-                                                          member_updates})
+        store = {
+            constants.MEMBER: member,
+            constants.LISTENERS: listeners,
+            constants.LOADBALANCER: load_balancer,
+            constants.POOL: pool,
+            constants.UPDATE_DICT: member_updates}
+        if load_balancer.availability_zone:
+            store[constants.AVAILABILITY_ZONE] = (
+                self._az_repo.get_availability_zone_metadata_dict(
+                    db_apis.get_session(), load_balancer.availability_zone))
+        else:
+            store[constants.AVAILABILITY_ZONE] = {}
+
+        update_member_tf = self._taskflow_load(
+            self._member_flows.get_update_member_flow(),
+            store=store)
         with tf_logging.DynamicLoggingListener(update_member_tf,
                                                log=LOG):
             update_member_tf.run()
@@ -862,6 +901,7 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
             amp_failover_flow = self._amphora_flows.get_failover_amphora_flow(
                 amphora, lb_amp_count)
 
+            az_metadata = {}
             flavor = {}
             lb_id = None
             vip = None
@@ -876,11 +916,17 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
                 else:
                     flavor = {constants.LOADBALANCER_TOPOLOGY:
                               loadbalancer.topology}
+                if loadbalancer.availability_zone:
+                    az_metadata = (
+                        self._az_repo.get_availability_zone_metadata_dict(
+                            db_apis.get_session(),
+                            loadbalancer.availability_zone))
                 vip = loadbalancer.vip
                 server_group_id = loadbalancer.server_group_id
 
-            stored_params = {constants.BUILD_TYPE_PRIORITY:
-                             constants.LB_CREATE_FAILOVER_PRIORITY,
+            stored_params = {constants.AVAILABILITY_ZONE: az_metadata,
+                             constants.BUILD_TYPE_PRIORITY:
+                                 constants.LB_CREATE_FAILOVER_PRIORITY,
                              constants.FLAVOR: flavor,
                              constants.LOADBALANCER: loadbalancer,
                              constants.SERVER_GROUP_ID: server_group_id,
@@ -1031,6 +1077,13 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
                              constants.LOADBALANCER_ID: lb.id,
                              constants.FLAVOR: flavor}
 
+            if lb.availability_zone:
+                stored_params[constants.AVAILABILITY_ZONE] = (
+                    self._az_repo.get_availability_zone_metadata_dict(
+                        db_apis.get_session(), lb.availability_zone))
+            else:
+                stored_params[constants.AVAILABILITY_ZONE] = {}
+
             failover_lb_tf = self._taskflow_load(lb_failover_flow,
                                                  store=stored_params)
 
@@ -1057,8 +1110,7 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
 
         amp = self._amphora_repo.get(db_apis.get_session(),
                                      id=amphora_id)
-        LOG.info("Start amphora cert rotation, amphora's id is: %s",
-                 amphora_id)
+        LOG.info("Start amphora cert rotation, amphora's id is: %s", amp.id)
 
         certrotation_amphora_tf = self._taskflow_load(
             self._amphora_flows.cert_rotate_amphora_flow(),
@@ -1068,8 +1120,6 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         with tf_logging.DynamicLoggingListener(certrotation_amphora_tf,
                                                log=LOG):
             certrotation_amphora_tf.run()
-        LOG.info("Finished amphora cert rotation, amphora's id was: %s",
-                 amphora_id)
 
     def update_amphora_agent_config(self, amphora_id):
         """Update the amphora agent configuration.
@@ -1099,5 +1149,3 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         with tf_logging.DynamicLoggingListener(update_amphora_tf,
                                                log=LOG):
             update_amphora_tf.run()
-        LOG.info("Finished amphora agent configuration update, amphora's id "
-                 "was: %s", amphora_id)
