@@ -56,9 +56,6 @@ class AmpListenersUpdate(BaseAmphoraTask):
         # in a failover flow with both amps failing. Skip it and let
         # health manager fix it.
         try:
-            # Make sure we have a fresh load balancer object
-            loadbalancer = self.loadbalancer_repo.get(db_apis.get_session(),
-                                                      id=loadbalancer.id)
             self.amphora_driver.update_amphora_listeners(
                 loadbalancer, amphora, timeout_dict)
         except Exception as e:
@@ -78,9 +75,6 @@ class AmphoraIndexListenerUpdate(BaseAmphoraTask):
         # in a failover flow with both amps failing. Skip it and let
         # health manager fix it.
         try:
-            # Make sure we have a fresh load balancer object
-            loadbalancer = self.loadbalancer_repo.get(db_apis.get_session(),
-                                                      id=loadbalancer.id)
             self.amphora_driver.update_amphora_listeners(
                 loadbalancer, amphorae[amphora_index], timeout_dict)
         except Exception as e:
@@ -282,7 +276,7 @@ class AmphoraCertUpload(BaseAmphoraTask):
     def execute(self, amphora, server_pem):
         """Execute cert_update_amphora routine."""
         LOG.debug("Upload cert in amphora REST driver")
-        key = utils.get_six_compatible_server_certs_key_passphrase()
+        key = utils.get_compatible_server_certs_key_passphrase()
         fer = fernet.Fernet(key)
         self.amphora_driver.upload_cert_amp(amphora, fer.decrypt(server_pem))
 
@@ -383,15 +377,6 @@ class AmphoraIndexVRRPUpdate(BaseAmphoraTask):
         LOG.debug("Uploaded VRRP configuration of amphora %s.", amphora_id)
 
 
-class AmphoraVRRPStop(BaseAmphoraTask):
-    """Task to stop keepalived of all amphorae of a LB."""
-
-    def execute(self, loadbalancer):
-        self.amphora_driver.stop_vrrp_service(loadbalancer)
-        LOG.debug("Stopped VRRP of loadbalancer %s amphorae",
-                  loadbalancer.id)
-
-
 class AmphoraVRRPStart(BaseAmphoraTask):
     """Task to start keepalived on an amphora.
 
