@@ -12,14 +12,11 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 #
-from __future__ import division
-
 import time
 
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import excutils
-import six
 from taskflow import task
 from taskflow.types import failure
 import tenacity
@@ -56,12 +53,18 @@ class CalculateAmphoraDelta(BaseNetworkTask):
 
     default_provides = constants.DELTA
 
-    def execute(self, loadbalancer, amphora, vrrp_port=None):
+    def execute(self, loadbalancer, amphora, availability_zone,
+                vrrp_port=None):
         LOG.debug("Calculating network delta for amphora id: %s", amphora.id)
 
         if vrrp_port is None:
             vrrp_port = self.network_driver.get_port(amphora.vrrp_port_id)
-        management_nets = CONF.controller_worker.amp_boot_network_list
+        if availability_zone:
+            management_nets = (
+                [availability_zone.get(constants.MANAGEMENT_NETWORK)] or
+                CONF.controller_worker.amp_boot_network_list)
+        else:
+            management_nets = CONF.controller_worker.amp_boot_network_list
         desired_network_ids = {vrrp_port.network_id}.union(management_nets)
 
         for pool in loadbalancer.pools:
@@ -99,24 +102,27 @@ class CalculateDelta(BaseNetworkTask):
 
     default_provides = constants.DELTAS
 
-    def execute(self, loadbalancer):
+    def execute(self, loadbalancer, availability_zone):
         """Compute which NICs need to be plugged
 
         for the amphora to become operational.
 
         :param loadbalancer: the loadbalancer to calculate deltas for all
                              amphorae
+        :param availability_zone: availability zone metadata dict
+
         :returns: dict of octavia.network.data_models.Delta keyed off amphora
                   id
         """
 
         calculate_amp = CalculateAmphoraDelta()
         deltas = {}
-        for amphora in six.moves.filter(
+        for amphora in filter(
             lambda amp: amp.status == constants.AMPHORA_ALLOCATED,
                 loadbalancer.amphorae):
 
-            delta = calculate_amp.execute(loadbalancer, amphora)
+            delta = calculate_amp.execute(loadbalancer, amphora,
+                                          availability_zone)
             deltas[amphora.id] = delta
         return deltas
 
@@ -281,7 +287,7 @@ class HandleNetworkDeltas(BaseNetworkTask):
     def execute(self, deltas):
         """Handle network plugging based off deltas."""
         added_ports = {}
-        for amp_id, delta in six.iteritems(deltas):
+        for amp_id, delta in deltas.items():
             added_ports[amp_id] = []
             for nic in delta.add_nics:
                 interface = self.network_driver.plug_network(delta.compute_id,
@@ -307,7 +313,7 @@ class HandleNetworkDeltas(BaseNetworkTask):
 
         if isinstance(result, failure.Failure):
             return
-        for amp_id, delta in six.iteritems(deltas):
+        for amp_id, delta in deltas.items():
             LOG.warning("Unable to plug networks for amp id %s",
                         delta.amphora_id)
             if not delta:
@@ -344,7 +350,7 @@ class PlugVIP(BaseNetworkTask):
         try:
             # Make sure we have the current port IDs for cleanup
             for amp_data in result:
-                for amphora in six.moves.filter(
+                for amphora in filter(
                         # pylint: disable=cell-var-from-loop
                         lambda amp: amp.id == amp_data.id,
                         loadbalancer.amphorae):
@@ -462,20 +468,6 @@ class AllocateVIP(BaseNetworkTask):
                       {'vip': vip.ip_address, 'except': e})
 
 
-class AllocateVIPforFailover(AllocateVIP):
-    """Task to allocate/validate the VIP for a failover flow."""
-
-    def revert(self, result, loadbalancer, *args, **kwargs):
-        """Handle a failure to allocate vip."""
-
-        if isinstance(result, failure.Failure):
-            LOG.exception("Unable to allocate VIP")
-            return
-        vip = result
-        LOG.info("Failover revert is not deallocating vip %s because this is "
-                 "a failover.", vip.ip_address)
-
-
 class DeallocateVIP(BaseNetworkTask):
     """Task to deallocate a VIP."""
 
