@@ -27,6 +27,7 @@ from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_utils import excutils
 from oslo_utils import uuidutils
+from sqlalchemy.orm import joinedload
 from sqlalchemy.orm import noload
 from sqlalchemy.orm import subqueryload
 from sqlalchemy.sql.expression import false
@@ -106,7 +107,7 @@ class BaseRepository(object):
         """
         with session.begin(subtransactions=True):
             tags = model_kwargs.pop('tags', None)
-            if tags is not None:
+            if tags:
                 resource = session.query(self.model_class).get(id)
                 resource.tags = tags
             session.query(self.model_class).filter_by(
@@ -229,6 +230,8 @@ class Repositories(object):
         self.flavor = FlavorRepository()
         self.flavor_profile = FlavorProfileRepository()
         self.spares_pool = SparesPoolRepository()
+        self.availability_zone = AvailabilityZoneRepository()
+        self.availability_zone_profile = AvailabilityZoneProfileRepository()
 
     def create_load_balancer_and_vip(self, session, lb_dict, vip_dict):
         """Inserts load balancer and vip entities into the database.
@@ -1061,12 +1064,12 @@ class ListenerRepository(BaseRepository):
 
     def update(self, session, id, **model_kwargs):
         with session.begin(subtransactions=True):
-            listener_db = session.query(self.model_class).filter_by(
-                id=id).first()
             tags = model_kwargs.pop('tags', None)
-            if tags is not None:
+            if tags:
                 resource = session.query(self.model_class).get(id)
                 resource.tags = tags
+            listener_db = session.query(self.model_class).filter_by(
+                id=id).first()
             # Verify any newly specified default_pool_id exists
             default_pool_id = model_kwargs.get('default_pool_id')
             if default_pool_id:
@@ -1189,7 +1192,8 @@ class AmphoraRepository(BaseRepository):
             load_balancer.amphorae.append(amphora)
 
     @oslo_db_api.wrap_db_retry(max_retries=5, retry_on_deadlock=True)
-    def allocate_and_associate(self, session, load_balancer_id):
+    def allocate_and_associate(self, session, load_balancer_id,
+                               availability_zone=None):
         """Allocate an amphora for a load balancer.
 
         For v0.5 this is simple, find a free amp and
@@ -1200,13 +1204,23 @@ class AmphoraRepository(BaseRepository):
         :param load_balancer_id: The load balancer id to associate
         :returns: The amphora ID for the load balancer or None
         """
+        filters = {
+            'status': 'READY',
+            'load_balancer_id': None
+        }
+        if availability_zone:
+            LOG.debug("Filtering amps by zone: %s", availability_zone)
+            filters['cached_zone'] = availability_zone
+
         with session.begin(subtransactions=True):
             amp = session.query(self.model_class).with_for_update().filter_by(
-                status='READY', load_balancer_id=None).first()
+                **filters).first()
 
             if amp is None:
                 return None
 
+            if availability_zone:
+                LOG.debug("Found amp: %s in %s", amp.id, amp.cached_zone)
             amp.status = 'ALLOCATED'
             amp.load_balancer_id = load_balancer_id
 
@@ -1233,19 +1247,39 @@ class AmphoraRepository(BaseRepository):
                 .filter(models.Amphora.status != consts.DELETED)
                 # And the LB is also not DELETED
                 .filter(models.LoadBalancer.provisioning_status !=
-                        consts.DELETED)).first()
+                        consts.DELETED)
+                # And what does this do? Some SQLAlchemy magic?
+                .options(joinedload('*'))
+            ).first()
             if db_lb:
                 return db_lb.to_data_model()
             return None
 
-    def get_spare_amphora_count(self, session):
+    def get_spare_amphora_count(self, session, availability_zone=None,
+                                check_booting_amphora=False):
         """Get the count of the spare amphora.
 
         :returns: Number of current spare amphora.
         """
+        filters = {
+            'load_balancer_id': None
+        }
+        # For jobboard based controller amphora in booting/pending create state
+        # can reach READY state after restart of housekeeping/worker service,
+        # so include amphora in these state to query
+        if check_booting_amphora:
+            status = [consts.AMPHORA_READY,
+                      consts.AMPHORA_BOOTING,
+                      consts.PENDING_CREATE]
+        else:
+            status = [consts.AMPHORA_READY]
+
+        if availability_zone is not None:
+            filters['cached_zone'] = availability_zone
+
         with session.begin(subtransactions=True):
             count = session.query(self.model_class).filter_by(
-                status=consts.AMPHORA_READY, load_balancer_id=None).count()
+                **filters).filter(self.model_class.status.in_(status)).count()
 
         return count
 
@@ -1826,7 +1860,42 @@ class QuotasRepository(BaseRepository):
             session.flush()
 
 
-class FlavorRepository(BaseRepository):
+class _GetALLExceptDELETEDIdMixin(object):
+
+    def get_all(self, session, pagination_helper=None,
+                query_options=None, **filters):
+
+        """Retrieves a list of entities from the database.
+
+        This filters the "DELETED" placeholder from the list.
+
+        :param session: A Sql Alchemy database session.
+        :param pagination_helper: Helper to apply pagination and sorting.
+        :param query_options: Optional query options to apply.
+        :param filters: Filters to decide which entities should be retrieved.
+        :returns: [octavia.common.data_model]
+        """
+        query = session.query(self.model_class).filter_by(**filters)
+        if query_options:
+            query = query.options(query_options)
+
+        if hasattr(self.model_class, 'id'):
+            query = query.filter(self.model_class.id != consts.NIL_UUID)
+        else:
+            query = query.filter(self.model_class.name != consts.NIL_UUID)
+
+        if pagination_helper:
+            model_list, links = pagination_helper.apply(
+                query, self.model_class)
+        else:
+            links = None
+            model_list = query.all()
+
+        data_model_list = [model.to_data_model() for model in model_list]
+        return data_model_list, links
+
+
+class FlavorRepository(_GetALLExceptDELETEDIdMixin, BaseRepository):
     model_class = models.Flavor
 
     def get_flavor_metadata_dict(self, session, flavor_id):
@@ -1848,8 +1917,26 @@ class FlavorRepository(BaseRepository):
                     .filter(models.Flavor.flavor_profile_id ==
                             models.FlavorProfile.id).one()[0])
 
+    def delete(self, serial_session, **filters):
+        """Sets DELETED LBs flavor_id to NIL_UUID, then removes the flavor
 
-class FlavorProfileRepository(BaseRepository):
+        :param serial_session: A Sql Alchemy database transaction session.
+        :param filters: Filters to decide which entity should be deleted.
+        :returns: None
+        :raises: odb_exceptions.DBReferenceError
+        :raises: sqlalchemy.orm.exc.NoResultFound
+        """
+        (serial_session.query(models.LoadBalancer).
+         filter(models.LoadBalancer.flavor_id == filters['id']).
+         filter(models.LoadBalancer.provisioning_status == consts.DELETED).
+         update({models.LoadBalancer.flavor_id: consts.NIL_UUID},
+                synchronize_session=False))
+        flavor = (serial_session.query(self.model_class).
+                  filter_by(**filters).one())
+        serial_session.delete(flavor)
+
+
+class FlavorProfileRepository(_GetALLExceptDELETEDIdMixin, BaseRepository):
     model_class = models.FlavorProfile
 
 
@@ -1865,3 +1952,68 @@ class SparesPoolRepository(BaseRepository):
         """
         row = lock_session.query(models.SparesPool).with_for_update().one()
         return row
+
+
+class AvailabilityZoneRepository(_GetALLExceptDELETEDIdMixin, BaseRepository):
+    model_class = models.AvailabilityZone
+
+    def get_availability_zone_metadata_dict(self, session,
+                                            availability_zone_name):
+        with session.begin(subtransactions=True):
+            availability_zone_metadata_json = (
+                session.query(
+                    models.AvailabilityZoneProfile.availability_zone_data)
+                .filter(models.AvailabilityZone.name == availability_zone_name)
+                .filter(models.AvailabilityZone.availability_zone_profile_id ==
+                        models.AvailabilityZoneProfile.id)
+                .one()[0])
+            result_dict = (
+                {} if availability_zone_metadata_json is None
+                else jsonutils.loads(availability_zone_metadata_json))
+            return result_dict
+
+    def get_availability_zone_provider(self, session, availability_zone_name):
+        with session.begin(subtransactions=True):
+            return (session.query(models.AvailabilityZoneProfile.provider_name)
+                    .filter(
+                    models.AvailabilityZone.name == availability_zone_name)
+                    .filter(
+                    models.AvailabilityZone.availability_zone_profile_id ==
+                    models.AvailabilityZoneProfile.id).one()[0])
+
+    def update(self, session, name, **model_kwargs):
+        """Updates an entity in the database.
+
+        :param session: A Sql Alchemy database session.
+        :param model_kwargs: Entity attributes that should be updates.
+        :returns: octavia.common.data_model
+        """
+        with session.begin(subtransactions=True):
+            session.query(self.model_class).filter_by(
+                name=name).update(model_kwargs)
+
+    def delete(self, serial_session, **filters):
+        """Special delete method for availability_zone.
+
+        Sets DELETED LBs availability_zone to NIL_UUID, then removes the
+        availability_zone.
+
+        :param serial_session: A Sql Alchemy database transaction session.
+        :param filters: Filters to decide which entity should be deleted.
+        :returns: None
+        :raises: odb_exceptions.DBReferenceError
+        :raises: sqlalchemy.orm.exc.NoResultFound
+        """
+        (serial_session.query(models.LoadBalancer).
+         filter(models.LoadBalancer.availability_zone == filters[consts.NAME]).
+         filter(models.LoadBalancer.provisioning_status == consts.DELETED).
+         update({models.LoadBalancer.availability_zone: consts.NIL_UUID},
+                synchronize_session=False))
+        availability_zone = (
+            serial_session.query(self.model_class).filter_by(**filters).one())
+        serial_session.delete(availability_zone)
+
+
+class AvailabilityZoneProfileRepository(_GetALLExceptDELETEDIdMixin,
+                                        BaseRepository):
+    model_class = models.AvailabilityZoneProfile
