@@ -15,9 +15,9 @@
 import os
 import stat
 import subprocess
+from unittest import mock
 
 import flask
-import mock
 
 from oslo_config import cfg
 from oslo_config import fixture as oslo_fixture
@@ -42,7 +42,7 @@ class KeepalivedLvsTestCase(base.TestCase):
         "# Configuration for Listener %(listener_id)s\n\n"
         "net_namespace haproxy-amphora\n\n"
         "virtual_server 10.0.0.2 80 {\n"
-        "    lb_algo wrr\n"
+        "    lb_algo rr\n"
         "    lb_kind NAT\n"
         "    protocol udp\n"
         "    delay_loop 30\n"
@@ -171,10 +171,6 @@ class KeepalivedLvsTestCase(base.TestCase):
             m_fdopen.assert_any_call('TEST-WRITE-SYSINIT', 'w')
             self.assertEqual(200, res.status_code)
 
-    @mock.patch('octavia.amphorae.backends.agent.api_server.util.'
-                'get_udp_listeners')
-    @mock.patch('octavia.amphorae.backends.agent.api_server.util.'
-                'get_loadbalancers')
     @mock.patch('octavia.amphorae.backends.agent.api_server.util.'
                 'run_systemctl_command')
     @mock.patch('octavia.amphorae.backends.agent.api_server.util.'
@@ -191,10 +187,8 @@ class KeepalivedLvsTestCase(base.TestCase):
     def test_upload_udp_listener_config_with_vrrp_check_dir(
             self, m_check_output, m_os_rm, m_os_mkdir, m_exists, m_os_chmod,
             m_os_sysinit, m_copy2, mock_netns, mock_install_netns,
-            mock_systemctl, mock_get_lbs, mock_get_udp_listeners):
+            mock_systemctl):
         m_exists.side_effect = [False, False, True, True, False, False, False]
-        mock_get_lbs.return_value = []
-        mock_get_udp_listeners.return_value = [self.FAKE_ID]
         cfg_path = util.keepalived_lvs_cfg_path(self.FAKE_ID)
         m = self.useFixture(test_utils.OpenFixture(cfg_path)).mock_open
 
@@ -243,8 +237,6 @@ class KeepalivedLvsTestCase(base.TestCase):
             m_fdopen.assert_any_call('TEST-WRITE-CFG', 'wb')
             m_fdopen.assert_any_call('TEST-WRITE-SYSINIT', 'w')
             m_fdopen.assert_any_call('TEST-WRITE-UDP-VRRP-CHECK', 'w')
-
-            m_os_rm.assert_called_once_with(util.haproxy_check_script_path())
             self.assertEqual(200, res.status_code)
 
     @mock.patch('octavia.amphorae.backends.agent.api_server.util.'
