@@ -40,7 +40,7 @@ class TestLvsCfg(base.TestCase):
                "# Configuration for Listener sample_listener_id_1\n\n"
                "net_namespace amphora-haproxy\n\n"
                "virtual_server 10.0.0.2 80 {\n"
-               "    lb_algo wrr\n"
+               "    lb_algo rr\n"
                "    lb_kind NAT\n"
                "    protocol UDP\n"
                "    persistence_timeout 33\n"
@@ -86,7 +86,7 @@ class TestLvsCfg(base.TestCase):
                "# Configuration for Listener sample_listener_id_1\n\n"
                "net_namespace amphora-haproxy\n\n"
                "virtual_server 10.0.0.2 80 {\n"
-               "    lb_algo wrr\n"
+               "    lb_algo rr\n"
                "    lb_kind NAT\n"
                "    protocol UDP\n"
                "    delay_loop 30\n"
@@ -129,7 +129,7 @@ class TestLvsCfg(base.TestCase):
                "# Configuration for Listener sample_listener_id_1\n\n"
                "net_namespace amphora-haproxy\n\n"
                "virtual_server 10.0.0.2 80 {\n"
-               "    lb_algo wrr\n"
+               "    lb_algo rr\n"
                "    lb_kind NAT\n"
                "    protocol UDP\n"
                "    delay_loop 30\n"
@@ -172,7 +172,7 @@ class TestLvsCfg(base.TestCase):
                "# Configuration for Listener sample_listener_id_1\n\n"
                "net_namespace amphora-haproxy\n\n"
                "virtual_server 10.0.0.2 80 {\n"
-               "    lb_algo wrr\n"
+               "    lb_algo rr\n"
                "    lb_kind NAT\n"
                "    protocol UDP\n"
                "    delay_loop 30\n"
@@ -222,50 +222,6 @@ class TestLvsCfg(base.TestCase):
                 persistence=False, alloc_default_pool=False))
         self.assertEqual(exp, rendered_obj)
 
-    def test_render_template_udp_with_pool_no_member(self):
-        exp = ("# Configuration for Loadbalancer sample_loadbalancer_id_1\n"
-               "# Configuration for Listener sample_listener_id_1\n\n"
-               "net_namespace amphora-haproxy\n\n"
-               "virtual_server 10.0.0.2 80 {\n"
-               "    lb_algo wrr\n"
-               "    lb_kind NAT\n"
-               "    protocol UDP\n\n\n"
-               "    # Configuration for Pool sample_pool_id_0\n"
-               "}\n\n")
-
-        rendered_obj = self.udp_jinja_cfg.render_loadbalancer_obj(
-            sample_configs_combined.sample_listener_tuple(
-                proto=constants.PROTOCOL_UDP, monitor=False,
-                persistence=False, alloc_default_pool=True,
-                sample_default_pool=0))
-        self.assertEqual(exp, rendered_obj)
-
-    def test_render_template_udp_with_disabled_pool(self):
-        exp = ("# Configuration for Loadbalancer sample_loadbalancer_id_1\n"
-               "# Configuration for Listener sample_listener_id_1\n\n"
-               "net_namespace amphora-haproxy\n\n"
-               "virtual_server 10.0.0.2 80 {\n"
-               "    lb_algo wrr\n"
-               "    lb_kind NAT\n"
-               "    protocol UDP\n\n\n"
-               "    # Pool sample_pool_id_1 is disabled\n"
-               "    # Configuration for Member sample_member_id_1\n"
-               "    real_server 10.0.0.99 82 {\n"
-               "        weight 13\n\n"
-               "    }\n\n"
-               "    # Configuration for Member sample_member_id_2\n"
-               "    real_server 10.0.0.98 82 {\n"
-               "        weight 13\n\n"
-               "    }\n\n"
-               "}\n\n")
-
-        rendered_obj = self.udp_jinja_cfg.render_loadbalancer_obj(
-            sample_configs_combined.sample_listener_tuple(
-                proto=constants.PROTOCOL_UDP, monitor=False,
-                persistence=False, alloc_default_pool=True,
-                pool_enabled=False))
-        self.assertEqual(exp, rendered_obj)
-
     def test_udp_transform_session_persistence(self):
         persistence_src_ip = (
             sample_configs_combined.sample_session_persistence_tuple(
@@ -342,17 +298,106 @@ class TestLvsCfg(base.TestCase):
         sample_configs_combined.RET_UDP_LISTENER.pop('connection_limit')
         self.assertEqual(sample_configs_combined.RET_UDP_LISTENER, ret)
 
-    def test_render_template_disabled_udp_listener(self):
+    def test_render_template_udp_listener_with_http_health_monitor(self):
         exp = ("# Configuration for Loadbalancer sample_loadbalancer_id_1\n"
-               "# Listener sample_listener_id_1 is disabled\n\n"
-               "net_namespace amphora-haproxy\n\n")
-        rendered_obj = self.udp_jinja_cfg.render_loadbalancer_obj(
-            sample_configs_combined.sample_listener_tuple(
-                enabled=False,
-                proto=constants.PROTOCOL_UDP,
-                persistence_type=constants.SESSION_PERSISTENCE_SOURCE_IP,
-                persistence_timeout=33,
-                persistence_granularity='255.255.0.0',
-                monitor_proto=constants.HEALTH_MONITOR_UDP_CONNECT,
-                connection_limit=98))
+               "# Configuration for Listener sample_listener_id_1\n\n"
+               "net_namespace amphora-haproxy\n\n"
+               "virtual_server 10.0.0.2 80 {\n"
+               "    lb_algo rr\n"
+               "    lb_kind NAT\n"
+               "    protocol UDP\n"
+               "    delay_loop 30\n"
+               "    delay_before_retry 30\n"
+               "    retry 3\n\n\n"
+               "    # Configuration for Pool sample_pool_id_1\n"
+               "    # Configuration for HealthMonitor sample_monitor_id_1\n"
+               "    # Configuration for Member sample_member_id_1\n"
+               "    real_server 10.0.0.99 82 {\n"
+               "        weight 13\n"
+               "        uthreshold 98\n"
+               "        HTTP_GET {\n"
+               "            url {\n"
+               "              path /index.html\n"
+               "              status_code 200\n"
+               "            }\n"
+               "            url {\n"
+               "              path /index.html\n"
+               "              status_code 201\n"
+               "            }\n"
+               "            connect_ip 10.0.0.99\n"
+               "            connect_port 82\n"
+               "            connect_timeout 31\n"
+               "        }\n"
+               "    }\n\n"
+               "    # Configuration for Member sample_member_id_2\n"
+               "    real_server 10.0.0.98 82 {\n"
+               "        weight 13\n"
+               "        uthreshold 98\n"
+               "        HTTP_GET {\n"
+               "            url {\n"
+               "              path /index.html\n"
+               "              status_code 200\n"
+               "            }\n"
+               "            url {\n"
+               "              path /index.html\n"
+               "              status_code 201\n"
+               "            }\n"
+               "            connect_ip 10.0.0.98\n"
+               "            connect_port 82\n"
+               "            connect_timeout 31\n"
+               "        }\n"
+               "    }\n\n"
+               "}\n\n")
+
+        listener = sample_configs_combined.sample_listener_tuple(
+            proto=constants.PROTOCOL_UDP,
+            monitor_proto=constants.HEALTH_MONITOR_HTTP,
+            connection_limit=98,
+            persistence=False,
+            monitor_expected_codes='200-201')
+
+        rendered_obj = self.udp_jinja_cfg.render_loadbalancer_obj(listener)
+        self.assertEqual(exp, rendered_obj)
+
+    def test_render_template_udp_listener_with_tcp_health_monitor(self):
+        exp = ("# Configuration for Loadbalancer sample_loadbalancer_id_1\n"
+               "# Configuration for Listener sample_listener_id_1\n\n"
+               "net_namespace amphora-haproxy\n\n"
+               "virtual_server 10.0.0.2 80 {\n"
+               "    lb_algo rr\n"
+               "    lb_kind NAT\n"
+               "    protocol UDP\n"
+               "    delay_loop 30\n"
+               "    delay_before_retry 30\n"
+               "    retry 3\n\n\n"
+               "    # Configuration for Pool sample_pool_id_1\n"
+               "    # Configuration for HealthMonitor sample_monitor_id_1\n"
+               "    # Configuration for Member sample_member_id_1\n"
+               "    real_server 10.0.0.99 82 {\n"
+               "        weight 13\n"
+               "        uthreshold 98\n"
+               "        TCP_CHECK {\n"
+               "            connect_ip 10.0.0.99\n"
+               "            connect_port 82\n"
+               "            connect_timeout 31\n"
+               "        }\n"
+               "    }\n\n"
+               "    # Configuration for Member sample_member_id_2\n"
+               "    real_server 10.0.0.98 82 {\n"
+               "        weight 13\n"
+               "        uthreshold 98\n"
+               "        TCP_CHECK {\n"
+               "            connect_ip 10.0.0.98\n"
+               "            connect_port 82\n"
+               "            connect_timeout 31\n"
+               "        }\n"
+               "    }\n\n"
+               "}\n\n")
+        listener = sample_configs_combined.sample_listener_tuple(
+            proto=constants.PROTOCOL_UDP,
+            monitor_proto=constants.HEALTH_MONITOR_TCP,
+            connection_limit=98,
+            persistence=False)
+
+        rendered_obj = self.udp_jinja_cfg.render_loadbalancer_obj(listener)
         self.assertEqual(exp, rendered_obj)
