@@ -12,9 +12,9 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 #
+from unittest import mock
 
 from cryptography import fernet
-import mock
 from oslo_config import cfg
 from oslo_config import fixture as oslo_fixture
 from oslo_utils import uuidutils
@@ -83,9 +83,7 @@ class TestAmphoraDriverTasks(base.TestCase):
                              constants.CONN_RETRY_INTERVAL: 4}
         super(TestAmphoraDriverTasks, self).setUp()
 
-    @mock.patch('octavia.db.repositories.LoadBalancerRepository.get')
     def test_amp_listeners_update(self,
-                                  mock_lb_repo_get,
                                   mock_driver,
                                   mock_generate_uuid,
                                   mock_log,
@@ -94,13 +92,12 @@ class TestAmphoraDriverTasks(base.TestCase):
                                   mock_listener_repo_update,
                                   mock_amphora_repo_update):
 
-        mock_lb_repo_get.return_value = _LB_mock
         amp_list_update_obj = amphora_driver_tasks.AmpListenersUpdate()
         amp_list_update_obj.execute(_load_balancer_mock, _amphora_mock,
                                     self.timeout_dict)
 
         mock_driver.update_amphora_listeners.assert_called_once_with(
-            _LB_mock, _amphora_mock, self.timeout_dict)
+            _load_balancer_mock, _amphora_mock, self.timeout_dict)
 
         mock_driver.update_amphora_listeners.side_effect = Exception('boom')
 
@@ -110,24 +107,17 @@ class TestAmphoraDriverTasks(base.TestCase):
         mock_amphora_repo_update.assert_called_once_with(
             _session_mock, AMP_ID, status=constants.ERROR)
 
-    @mock.patch('octavia.db.repositories.LoadBalancerRepository.get')
-    def test_amphorae_listeners_update(self,
-                                       mock_lb_repo_get,
-                                       mock_driver,
-                                       mock_generate_uuid,
-                                       mock_log,
-                                       mock_get_session,
-                                       mock_listener_repo_get,
-                                       mock_listener_repo_update,
-                                       mock_amphora_repo_update):
+    def test_amphorae_listeners_update(
+            self, mock_driver, mock_generate_uuid, mock_log, mock_get_session,
+            mock_listener_repo_get, mock_listener_repo_update,
+            mock_amphora_repo_update):
 
-        mock_lb_repo_get.return_value = _LB_mock
         amp_list_update_obj = amphora_driver_tasks.AmphoraIndexListenerUpdate()
         amp_list_update_obj.execute(_load_balancer_mock, 0,
                                     [_amphora_mock], self.timeout_dict)
 
         mock_driver.update_amphora_listeners.assert_called_once_with(
-            _LB_mock, _amphora_mock, self.timeout_dict)
+            _load_balancer_mock, _amphora_mock, self.timeout_dict)
 
         mock_driver.update_amphora_listeners.side_effect = Exception('boom')
 
@@ -214,7 +204,7 @@ class TestAmphoraDriverTasks(base.TestCase):
 
         # Test no listeners
         mock_lb.listeners = None
-        listeners_reload_obj.execute(mock_lb, None, 0)
+        listeners_reload_obj.execute(mock_lb, 0, None)
         mock_driver.reload.assert_not_called()
 
         # Test with listeners
@@ -581,10 +571,10 @@ class TestAmphoraDriverTasks(base.TestCase):
                                  mock_listener_repo_get,
                                  mock_listener_repo_update,
                                  mock_amphora_repo_update):
-        key = utils.get_six_compatible_server_certs_key_passphrase()
+        key = utils.get_compatible_server_certs_key_passphrase()
         fer = fernet.Fernet(key)
         pem_file_mock = fer.encrypt(
-            utils.get_six_compatible_value('test-pem-file'))
+            utils.get_compatible_value('test-pem-file'))
         amphora_cert_upload_mock = amphora_driver_tasks.AmphoraCertUpload()
         amphora_cert_upload_mock.execute(_amphora_mock, pem_file_mock)
 
@@ -708,19 +698,6 @@ class TestAmphoraDriverTasks(base.TestCase):
         mock_amphora_repo_update.assert_called_once_with(
             _session_mock, _amphora_mock.id, status=constants.ERROR)
 
-    def test_amphora_vrrp_stop(self,
-                               mock_driver,
-                               mock_generate_uuid,
-                               mock_log,
-                               mock_get_session,
-                               mock_listener_repo_get,
-                               mock_listener_repo_update,
-                               mock_amphora_repo_update):
-        amphora_vrrp_stop_obj = (
-            amphora_driver_tasks.AmphoraVRRPStop())
-        amphora_vrrp_stop_obj.execute(_LB_mock)
-        mock_driver.stop_vrrp_service.assert_called_once_with(_LB_mock)
-
     def test_amphora_vrrp_start(self,
                                 mock_driver,
                                 mock_generate_uuid,
