@@ -11,9 +11,8 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-#
+from unittest import mock
 
-import mock
 from oslo_config import cfg
 from oslo_config import fixture as oslo_fixture
 from oslo_utils import uuidutils
@@ -122,7 +121,7 @@ class TestNetworkTasks(base.TestCase):
         calc_amp_delta = network_tasks.CalculateAmphoraDelta()
 
         # Test vrrp_port_id is None
-        result = calc_amp_delta.execute(lb_mock, amphora_mock)
+        result = calc_amp_delta.execute(lb_mock, amphora_mock, {})
 
         self.assertEqual(AMPHORA_ID, result.amphora_id)
         self.assertEqual(COMPUTE_ID, result.compute_id)
@@ -137,7 +136,7 @@ class TestNetworkTasks(base.TestCase):
         # Test with vrrp_port_id
         mock_driver.reset_mock()
 
-        result = calc_amp_delta.execute(lb_mock, amphora_mock,
+        result = calc_amp_delta.execute(lb_mock, amphora_mock, {},
                                         vrrp_port=vrrp_port_mock)
 
         self.assertEqual(AMPHORA_ID, result.amphora_id)
@@ -162,7 +161,8 @@ class TestNetworkTasks(base.TestCase):
 
         calc_delta = network_tasks.CalculateDelta()
 
-        self.assertEqual(EMPTY, calc_delta.execute(self.load_balancer_mock))
+        self.assertEqual(EMPTY,
+                         calc_delta.execute(self.load_balancer_mock, {}))
 
         # Test with one amp and no pools, nothing plugged
         # Delta should be empty
@@ -173,7 +173,7 @@ class TestNetworkTasks(base.TestCase):
         self.load_balancer_mock.pools = []
 
         self.assertEqual(empty_deltas,
-                         calc_delta.execute(self.load_balancer_mock))
+                         calc_delta.execute(self.load_balancer_mock, {}))
         mock_driver.get_plugged_networks.assert_called_once_with(COMPUTE_ID)
 
         # Pool mock should be configured explicitly for each test
@@ -184,7 +184,7 @@ class TestNetworkTasks(base.TestCase):
         # Delta should be empty
         pool_mock.members = []
         self.assertEqual(empty_deltas,
-                         calc_delta.execute(self.load_balancer_mock))
+                         calc_delta.execute(self.load_balancer_mock, {}))
 
         # Test with one amp and one pool and one member, nothing plugged
         # Delta should be one additional subnet to plug
@@ -201,7 +201,7 @@ class TestNetworkTasks(base.TestCase):
                                     data_models.Interface(network_id=2)],
                                 delete_nics=[])
         self.assertEqual({self.amphora_mock.id: ndm},
-                         calc_delta.execute(self.load_balancer_mock))
+                         calc_delta.execute(self.load_balancer_mock, {}))
 
         vrrp_port_call = mock.call(self.amphora_mock.vrrp_port_id)
         mock_driver.get_port.assert_has_calls([vrrp_port_call])
@@ -221,7 +221,7 @@ class TestNetworkTasks(base.TestCase):
             data_models.Interface(network_id=2)]
 
         self.assertEqual(empty_deltas,
-                         calc_delta.execute(self.load_balancer_mock))
+                         calc_delta.execute(self.load_balancer_mock, {}))
 
         # Test with one amp and one pool and one member, wrong network plugged
         # Delta should be one network to add and one to remove
@@ -239,7 +239,7 @@ class TestNetworkTasks(base.TestCase):
                                 delete_nics=[
                                     data_models.Interface(network_id=3)])
         self.assertEqual({self.amphora_mock.id: ndm},
-                         calc_delta.execute(self.load_balancer_mock))
+                         calc_delta.execute(self.load_balancer_mock, {}))
 
         # Test with one amp and one pool and no members, one network plugged
         # Delta should be one network to remove
@@ -254,7 +254,7 @@ class TestNetworkTasks(base.TestCase):
                                 delete_nics=[
                                     data_models.Interface(network_id=2)])
         self.assertEqual({self.amphora_mock.id: ndm},
-                         calc_delta.execute(self.load_balancer_mock))
+                         calc_delta.execute(self.load_balancer_mock, {}))
 
     def test_get_plumbed_networks(self, mock_get_net_driver):
         mock_driver = mock.MagicMock()
@@ -716,22 +716,6 @@ class TestNetworkTasks(base.TestCase):
         net.revert(vip_mock, LB)
         mock_driver.deallocate_vip.assert_called_once_with(vip_mock)
 
-    def test_allocate_vip_for_failover(self, mock_get_net_driver):
-        mock_driver = mock.MagicMock()
-        mock_get_net_driver.return_value = mock_driver
-        net = network_tasks.AllocateVIPforFailover()
-
-        mock_driver.allocate_vip.return_value = LB.vip
-
-        mock_driver.reset_mock()
-        self.assertEqual(LB.vip, net.execute(LB))
-        mock_driver.allocate_vip.assert_called_once_with(LB)
-
-        # revert
-        vip_mock = mock.MagicMock()
-        net.revert(vip_mock, LB)
-        mock_driver.deallocate_vip.assert_not_called()
-
     def test_deallocate_vip(self, mock_get_net_driver):
         mock_driver = mock.MagicMock()
         mock_get_net_driver.return_value = mock_driver
